/** Use "Refactor" for mapping classnames after package refactoring or evolution

	* WHEN TO USE, and WHAT IT CAN DO.
	* It is most useful when you have serialized objects, and then want to
	* migrant them to a new version of your application, but in the meantime the
	* classes have evolved, so that the old objects won't "fit".  You can now
	* refactor the serialized versions so they do fit, and you can read them back
	* in.
	*
	* Various implementations of this interface enable you to change the names of
	* classes, move them across packages, and rename whole packages.  And, of
	* course, you can write your own.

	* ClassRefactor
	* The "ClassRefactor" implementation maps between specific classnames, both
	* of which are fully qualified by their package.  Thus, you can use this to
	* rename a class, or to move it to another package.

	* PackageRefactor
	* "PackageRefactor" is a convenience class, which enables you to move
	* all the classes in a package into another.  You could do this with
	* ClassRefactor, but you would have to specify them all individually.
	* WARNING: if you map two packages to the same package (or if a package
	* already exists of the destination name), you can have two classes with
	* the same name, which is probably not what you wanted.  The standard
	* behaviour of the Java classloader is to always load the first one it
	* finds in the classpath - it will not signal any ambiguity.
	
	* Planned: AutoDiscoveryPackageRefactor
	* (Not written yet) for each class in a package listed in "oldPackages", JSX
	* will search for that class in a specified list of "newPackages".  This
	* enables you to leave it all to JSX - however, name clashes become more
	* likely now, and you may need to disambiguate them, by specifying individual
	* mappings as in ClassRefactor.  Note that when only some classes in an
	* "old package" are moving, you can arrange for the others to stay by also
	* listing that "old package" as a "new package".  Alternatively, you could
	* write your own, that would first check to see if a class in an "old
	* package" is actually still there; and if so, not move it.  But this would
	* not detect the case where you forgot to delete the old class files!
	**/


package JSX;
public interface Refactor {
	public String mapClassname(String packageQualifiedClassname);
}
