/** (c) Brendan Macmillan 2001, 2002, licensed under GNU's GPL 2 
		(If your project is not GPL, then a fee is due - see license on website)
    Website: http://www.csse.monash.edu.au/~bren/JSX
    Website: http://freshmeat.net/projects/jsx
    List (can read without subscribing, need to join to post):
      http://groups.yahoo.com/group/JSX-ideas/messages
		Commercial licensing enquiries only:
			bren@objectlight.com */


package JSX.magic;
import java.lang.reflect.*;
import java.io.*;

class MagicClass13 extends MagicClassI {

	private static Method allocateNewObject;
	private static Method setObjectFieldValue;
	private static Method setPrimitiveFieldValues;
	private static Method getFieldIDs;
	MagicClass13() {
		try {
			allocateNewObject = java.io.ObjectInputStream.class.getDeclaredMethod(
				"allocateNewObject",
				new Class[] {Class.class, Class.class});
			allocateNewObject.setAccessible(true);

			setObjectFieldValue = java.io.ObjectInputStream.class.getDeclaredMethod(
				"setObjectFieldValue",
				new Class[] {Object.class, long.class, Class.class, Object.class});
			setObjectFieldValue.setAccessible(true);

			setPrimitiveFieldValues = java.io.ObjectInputStream.class.getDeclaredMethod(
				"setPrimitiveFieldValues",
				new Class[] {Object.class, long[].class, char[].class, byte[].class});
			setPrimitiveFieldValues.setAccessible(true);

			getFieldIDs = java.io.ObjectStreamClass.class.getDeclaredMethod(
				"getFieldIDs",
				new Class[] {ObjectStreamField[].class, long[].class, long[].class});
			getFieldIDs.setAccessible(true);

		} catch (NoSuchMethodException noMethod) {
			throw new InternalError(noMethod.toString());
		}

		try {
			osc = (ObjectStreamClass) newInstance(ObjectStreamClass.class);
		} catch (NotSerializableException e) {	// Serializable check in newInstance
			e.printStackTrace();
		}
	}


/**
	* first non-serializable ancestor with a public no-arg constructor.
	* currentClass: is the actual runtime class we want to create
	* currclass: is the closest non-serializable ancestor.
	* NB: Object is non-serializable, always bottoms out.
	* old JSX can serialize an Object - differs from JOS...
	**/
	public Object newInstance(Class currentClass)
			throws NotSerializableException {

/*
		if (!Serializable.class.isAssignableFrom(currentClass))
			throw new NotSerializableException("'"+currentClass+"' is not Serializable - JSX *can* serialize such classes, despite them not implementing the 'Serializable' interface; but this feature is currently switched off (since version JSX1.0.4.2), as an experiment to see if it makes any difference.  If this is an annoyance to you, please post to the mailing list immediately:\n" +
			"  http://groups.yahoo.com/group/JSX-ideas/messages");
*/

		Class currclass;
		if (Serializable.class.isAssignableFrom(currentClass)) {
//JOS way:
			currclass = currentClass; 	//begin search for ancestor
			if (currclass != null) {
				while (Serializable.class.isAssignableFrom(currclass)) 
					currclass = currclass.getSuperclass();
			}
		} else {
//JSX way:
			currclass = Object.class;	//always use object
		}

//way common to both
 		Object[] argValues = {currentClass, currclass};
		try {
			return allocateNewObject.invoke(null, argValues);
		} catch (InvocationTargetException invokeE) {
			throw new InternalError(invokeE.toString());
		} catch (IllegalAccessException noAccess) {
			throw new InternalError();
		}
	}


	public void setObjectFieldValue(Object parent, Field f, Class type, Object value) {
		long[] primIDs = new long[0];
		long[] objIDs = new long[1]; //we know only one field, and it's an Object
		myGetFieldIDs(f, primIDs, objIDs);

		Object[] argValues = {parent, new Long(objIDs[0]), type, value};
		try {
			setObjectFieldValue.invoke(null, argValues);
		} catch (InvocationTargetException invokeE) {
			throw new InternalError(invokeE.toString());
		} catch (IllegalAccessException noAccess) {
			throw new InternalError();
		}
	}


	public void setPrimitiveFieldValues(Object parent, Field f, Object value) {
		long[] primIDs = new long[1];	//we know only one field, and it's primitive
		long[] objIDs = new long[0];
		ObjectStreamField osf = myGetFieldIDs(f, primIDs, objIDs);
//Wish: to set final primitive fields in 1.3 using primitive values directly
		char[] typecode = {osf.getTypeCode()};	//It already knows this
		
		ByteArrayOutputStream ba1 = new ByteArrayOutputStream();
		DataOutputStream ba = new DataOutputStream(ba1);
		try {
			switch (typecode[0]) {
				case 'B': ba.writeByte(((Byte)value).byteValue()); break; //byte
				case 'C': ba.writeChar(((Character)value).charValue()); break; //char
				case 'S': ba.writeShort(((Short)value).shortValue()); break; //short
				case 'I': ba.writeInt(((Integer)value).intValue()); break; //int
				case 'J': ba.writeLong(((Long)value).longValue()); break; //long
				case 'F': ba.writeFloat(((Float)value).floatValue()); break; //float
				case 'D': ba.writeDouble(((Double)value).doubleValue()); break; //double
				case 'Z': ba.writeBoolean(((Boolean)value).booleanValue()); break; //boolean
				case '[':	//an array...
				case 'L': //an object
				default:
					throw new InternalError("setPrimitiveFieldValues only does primitives");
			}
			ba.flush();
			ba.close();
		} catch (IOException byteArrayProblem) {
			throw new InternalError(byteArrayProblem.toString());
		}

		Object[] argValues = {parent, primIDs, typecode, ba1.toByteArray()};
		try {
			setPrimitiveFieldValues.invoke(null, argValues);
		} catch (InvocationTargetException invokeE) {
			throw new InternalError("MagicClass13.setPrimitiveFieldValues() failure");
		} catch (IllegalAccessException noAccess) {
			throw new InternalError();
		}
	}


	static protected ObjectStreamField myGetFieldIDs(Field f, long[] primIDs, long[] objIDs) {
		ObjectStreamField osf = null;
		try {
			osf = new ObjectStreamField(f.getName(), f.getType());
			Field osfField = ObjectStreamField.class.getDeclaredField("field");
			osfField.setAccessible(true);
			osfField.set(osf, f);
		} catch (NoSuchFieldException noField) {
			throw new InternalError();
		} catch (IllegalAccessException noAccess) {
			throw new InternalError();
		}
		ObjectStreamField[] fs = {osf};
		getFieldIDs(fs, primIDs, objIDs);
		return osf;	//in case needed - primitive uses, object ignores
	}


	static protected void getFieldIDs(ObjectStreamField[] fs, long[] primIDs, long[] objIDs) {
		try {
			Object[] argValues = {fs, primIDs, objIDs};
			getFieldIDs.invoke(null, argValues); //null because static
		} catch (InvocationTargetException invokeE) {
			throw new InternalError(invokeE.toString());
		} catch (IllegalAccessException noAccess) {
			throw new InternalError();
		}
	}

}
