SeeMeYUI.use("gallery-io-poller", "json", "io-queue", function(Y) {
	
	SessionsUpdater = function(sessionController) {
		this.sessionController = sessionController;
		
		this.poller = Y.io.poll(10000, "rest/sessions?participant-id=" + sessionController.getParticipantId(), {
			on : {
				success : this.processSessions
			},
			headers : { 
				'Accept': 'application/json',
				'If-Modified-Since': new Date(0)
			}, 
			context: this
		});
		this.poller.set('pauseInactive', true);
	};
	
	SessionsUpdater.prototype = {
		start : function() {
			this.poller.start();
		},
		
		stop : function() {
			this.poller.stop();
		},
		
		processSessions : function(txId, r) {
			if (r.status != 200) {
				return;
			}
			
			var sessions = Y.JSON.parse(r.responseText);
			this.sessionController.updateSessions(sessions);
		}
	};
	
	SessionUpdater = function(sessionController) {
		this.sessionController = sessionController;
		
		var session = sessionController.getCurrentSession();
		this.poller = Y.io.poll(1000, "rest/sessions/" + session.id + "?participant-id=" + sessionController.getParticipantId(), {
			on : {
				success : this.processSession,
				failure : this.stop
			},
			headers : { 
				'Accept': 'application/json',
				'If-Modified-Since': new Date(0)
			}, 
			context: this
		});
		this.poller.set('pauseInactive', true);
	};
	
	SessionUpdater.prototype = {
		start : function(session) {
			this.poller.start();
		},
		
		stop : function() {
			if (this.poller) {
				this.poller.stop();
			}
		},
		
		processSession : function(txId, r) {
			if (r.status != 200) {
				return;
			}
			
			var session = Y.JSON.parse(r.responseText);
			this.sessionController.updateSession(session);
		}
	};
	
	sendRegisterToSession = function(session, participantId) {
		var cfg = {
			method: 'PUT',
			headers: { 
				'Content-Type': 'text/plain' 
			},
			data: participantId,
			context: this
		};
		
		Y.io.queue.stop();
		Y.io.queue("rest/sessions/" + session.id + "/clients", cfg);
		Y.io.queue.start();
	};
	
	sendUpdateComment = function(sessionId, topicId, cardId, text, editorId) {
		var json = Y.JSON.stringify({
			text: text,
			'client-id': editorId
		});
		var cfg = {
			method: 'PUT',
			headers: { 'Content-Type': 'application/json' },
			data: json
		};
		
		Y.io.queue.stop();
		Y.io.queue("rest/sessions/" + sessionId + "/topics/" + topicId + "/cards/" + cardId + "/comment", cfg);
		Y.io.queue.start();
	};
});