YUI.add("topic-view", function (Y) {

	Y.TopicView = Y.Base.create('topic-view', Y.View, [], {

		container: '<div class="topic-item" />',

		template: Y.one("#topic-template").getContent(),

		events: {},

		initializer: function () {
			var model = this.get('model'),
				modelId = model.get('id'),
				strings = Y.merge(model.getAttrs([ 'topic' ]), {
					id: modelId
				}),
				node = Y.Node.create(Y.substitute(this.template, strings));

			model.after('topic-model:viewingChange', this.updateBody, this);
			model.after('topic-model:topicChange', this.updateTopicText, this);
			
			node.one('#topic-head-' + modelId).after('click', this.toggle, this);

			var promptsView = new Y.PromptsView({
				container: node.one('#topic-prompts-container-' + modelId),
				model: model
			});
			this.set('promptsView', promptsView);

			var cardEditorView = new Y.CardEditorView({
				container: node.one('#topic-card-editor-container-' + modelId),
				model: model
			});
			this.set('cardEditorView', cardEditorView);

			var cardHistoryView = new Y.CardHistoryView({
				container: node.one('#topic-card-history-container-' + modelId),
				model: model
			});
			this.set('cardHistoryView', cardHistoryView);

			this.get('container').append(node);
			this._node = node;
		},

		render: function () {
			var container = this.get('container'),
				model = this.get('model'),
				modelId = model.get('id'),
				body = container.one('#topic-body-' + modelId),
				toggle = container.one('#topic-toggle-' + modelId);

			this.get('promptsView').render();
			this.get('cardEditorView').render();
			this.get('cardHistoryView').render();
			
			this.updateBody();

			return this;
		},
		
		destroy: function() {
			this._node.remove();
		},

		// -- Event Handlers -------------------------------------------------------
		
		updateBody: function() {
			var container = this.get('container'),
				model = this.get('model'),
				modelId = model.get('id'),
				body = container.one('#topic-body-' + modelId),
				toggle = container.one('#topic-toggle-' + modelId);
			
			if (model.get('viewing')) {
				body.show();
				toggle.replaceClass('closed', 'open');
			} else {
				body.hide();
				toggle.replaceClass('open', 'closed');
			}
		},
		
		updateTopicText: function(e) {
			var container = this.get('container'),
				model = this.get('model'),
				modelId = model.get('id'),
				label = container.one('#topic-label-' + modelId);
			
			label.set('innerHTML', model.get('topic'));
		},

		toggle: function(e) {
			this.get('model').toggleViewing();
		}
	});

}, "1.0");
