SeeMeYUI.use("datasource-polling", "datasource-io", "json", "gallery-accordion", "console", "node", "io-base", function(Y) {

	TopicController = function(sessionController, session) {
		this.sessionController = sessionController;
		this.session = session;

		this.accordion = new Y.Accordion({
			srcNode : "#brainstormings-accordion",
			useAnimation : true,
			collapseOthersOnExpand : true
		});

		this.accordion.on("beforeItemExpand", this.onBeforeAccordionItemExpand, this);
		this.accordion.on("beforeItemCollapse", this.onBeforeAccordionItemCollapse, this);
		this.accordion.on("itemExpanded", this.onAccordionItemExpandend, this);
		this.accordion.on("itemCollapsed", this.onAccordionItemCollapsed, this);
		this.accordion.on("itemAdded", this.onAccordionItemAdded, this);
		this.accordion.render();

		this.topics = [];

		this.closingItem = false;
		this.openingItem = false;
		this.ignoreItemEvent = [];
	};

	TopicController.prototype = {

		getSession : function() {
			return this.session;
		},

		getSessionController : function() {
			return this.sessionController;
		},

		getCurrentTopic : function() {
			return this.currentTopic;
		},

		createTopicNode : function(data) {
			var topic = new Topic(data, this);
			this.accordion.addItem(topic.getAccordionItem());
			return topic;
		},

		onAccordionItemAdded : function(e) {
			this.ignoreItemEvent[this.ignoreItemEvent.length] = e.item;
		},

		onBeforeAccordionItemExpand : function(e) {
			this.openingItem = true;
		},

		onBeforeAccordionItemCollapse : function(e) {
			this.closingItem = true;
		},

		onAccordionItemExpandend : function(e) {
			var topic = this.findTopicForAccordionItem(e.item);
			if (topic == null) { return; }

			this.currentTopic = topic;
			topic.focus();

			if (topic.isProposed()) {
				topic.setProposed(false);
			}

			this.openingItem = false;
		},

		onAccordionItemCollapsed : function(e) {
			var topic = this.findTopicForAccordionItem(e.item);
			if (topic == null) { return; }

			topic.setProposed(false);
			if (this.currentTopic == topic) {
				delete this.currentTopic;
			}

			if (!this.ignoreItem(e.item) && !this.openingItem) {
				topic.lostFocus();
			}
			this.closingItem = false;
		},

		ignoreItem : function(i) {
			if (this.ignoreItemEvent.length == 0) { return false; }

			var ni = [];
			var ignore = false;
			for ( var j = 0; j < this.ignoreItemEvent.length; j++) {
				if (this.ignoreItemEvent[j] == i) {
					ignore = true;
				} else {
					ni[ni.length] = this.ignoreItemEvent[j];
				}
			}
			this.ignoreItemEvent = ni;
			return ignore;
		},

		findTopicForAccordionItem : function(item) {
			for ( var i = 0; i < this.topics.length; i++) {
				if (this.topics[i].getAccordionItem() == item) { return this.topics[i]; }
			}
			return null;
		},

		update : function(data) {
			var topics = data;
			var newTopics = [];
			var found = 0;
			for (var i = 0; i < topics.length; i++) {
				var topic = this.findTopic(topics[i].id);
				if (topic == null) {
					topic = this.createTopicNode(topics[i]);
					created = true;
				} else {
					topic.update(topics[i]);
					found++;
				}
				newTopics[newTopics.length] = topic;
			}

			var oldTopics = this.topics;
			this.topics = newTopics;

			if (found != oldTopics.length) {
				// delete
				for (var i = 0; i < oldTopics.length; i++) {
					var topic = oldTopics[i];
					if (this.findTopic(topic.getData().id) == null) {
						this.accordion.removeItem(topic.getAccordionItem());
					}
				}
			}
		},

		findTopic : function(id) {
			for ( var i = 0; i < this.topics.length; i++) {
				if (this.topics[i].getId() == id) { return this.topics[i]; }
			}
			return null;
		},

		openTopic : function(topic) {
			var t = this.findTopic(topic.getId());

			if (!t) { return; }

			this.currentTopic = t;

			t.getAccordionItem().set("expanded", true);
			t.focus();
		},

		start : function() {
			Y.one("#brainstormings-accordion").show();
		},

		stop : function() {
			Y.one("#brainstormings-accordion").hide();
		},

		show : function() {
			Y.one("#brainstormings-accordion").show();
		},

		hide : function() {
			Y.one("#brainstormings-accordion").hide();
		},

		findTopicForPrompt : function(promptId) {
			for ( var i = 0; i < this.topics.length; i++) {
				var topic = this.topics[i];
				if (topic.getPromptById(promptId)) { return topic; }
			}

			return null;
		},

		showPrompt : function(prompt) {
			var topic = this.findTopicForPrompt(prompt.id);
			if (!topic) { return; }

			this.showPromptInTopic(prompt, topic);
		},

		showPromptInTopic : function(prompt, topic) {
			if (!this.isCurrentTopic(topic)) {
				this.openTopic(topic);
			}

			topic.showPrompt(prompt);
		},

		isCurrentTopic : function(topic) {
			return this.currentTopic && (this.currentTopic == topic);
		},

		setCurrentTopicView : function(current) {
			if (!current) { return; }
			if (this.openingItem) { return; }

			var topic = this.findTopic(current.topicId);
			this.openTopic(topic);

			if (current.promptId) {
				var prompt = topic.getPromptById(current.promptId);
				this.showPromptInTopic(prompt, topic);
			}
		}
	};
});
